@@ -1,9 +1,9 @@
 /*
- *  $Id: formbox.c,v 1.109 2025/01/09 22:33:20 tom Exp $
+ *  $Id: formbox.c,v 1.91 2016/08/28 13:50:58 tom Exp $
  *
  *  formbox.c -- implements the form (i.e., some pairs label/editbox)
  *
- *  Copyright 2003-2024,2025	Thomas E. Dickey
+ *  Copyright 2003-2013,2016	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -24,7 +24,7 @@
  *	Valery Reznic (valery_reznic@users.sourceforge.net)
  */
 
-#include <dlg_internals.h>
+#include <dialog.h>
 #include <dlg_keys.h>
 
 #define LLEN(n) ((n) * FORMBOX_TAGS)
@@ -40,13 +40,13 @@
 #define ItemHelp(i)     (dialog_vars.item_help ? items[LLEN(i) + 8] : dlg_strempty())
 
 static bool
-is_readonly(const DIALOG_FORMITEM * item)
+is_readonly(DIALOG_FORMITEM * item)
 {
     return ((item->type & 2) != 0) || (item->text_flen <= 0);
 }
 
 static bool
-is_hidden(const DIALOG_FORMITEM * item)
+is_hidden(DIALOG_FORMITEM * item)
 {
     return ((item->type & 1) != 0);
 }
@@ -75,7 +75,7 @@ move_past(WINDOW *win, int y, int x)
  * Print form item
  */
 static int
-print_item(WINDOW *win, const DIALOG_FORMITEM * item, int scrollamt, bool choice)
+print_item(WINDOW *win, DIALOG_FORMITEM * item, int scrollamt, bool choice)
 {
     int count = 0;
     int len;
@@ -151,13 +151,12 @@ static int
 set_choice(DIALOG_FORMITEM item[], int choice, int item_no, bool * noneditable)
 {
     int result = -1;
+    int i;
 
     *noneditable = FALSE;
     if (!is_readonly(&item[choice])) {
 	result = choice;
     } else {
-	int i;
-
 	for (i = 0; i < item_no; i++) {
 	    if (!is_readonly(&(item[i]))) {
 		result = i;
@@ -180,7 +179,7 @@ form_limit(DIALOG_FORMITEM item[])
 {
     int n;
     int limit = 0;
-    for (n = 0; item[n].name != NULL; ++n) {
+    for (n = 0; item[n].name != 0; ++n) {
 	if (limit < item[n].name_y)
 	    limit = item[n].name_y;
 	if (limit < item[n].text_y)
@@ -293,6 +292,7 @@ scroll_next(WINDOW *win, DIALOG_FORMITEM item[], int stepsize, int *choice, int
     int old_scroll = *scrollamt;
     int old_row = MIN(item[old_choice].text_y, item[old_choice].name_y);
     int target = old_scroll + stepsize;
+    int n;
 
     if (stepsize < 0) {
 	if (old_row != old_scroll)
@@ -309,9 +309,7 @@ scroll_next(WINDOW *win, DIALOG_FORMITEM item[], int stepsize, int *choice, int
     }
 
     if (result) {
-	int n;
-
-	for (n = 0; item[n].name != NULL; ++n) {
+	for (n = 0; item[n].name != 0; ++n) {
 	    if (item[n].text_flen > 0) {
 		int new_row = MIN(item[n].text_y, item[n].name_y);
 		if (abs(new_row - target) < abs(old_row - target)) {
@@ -394,6 +392,7 @@ make_FORM_ELTs(DIALOG_FORMITEM * item,
 	    sprintf(item[i].text, "%.*s", item[i].text_ilen, old_text);
 
 	    if (item[i].text_free) {
+		item[i].text_free = FALSE;
 		free(old_text);
 	    }
 	    item[i].text_free = TRUE;
@@ -409,9 +408,9 @@ dlg_default_formitem(DIALOG_FORMITEM * items)
 {
     int result = 0;
 
-    if (dialog_vars.default_item != NULL) {
+    if (dialog_vars.default_item != 0) {
 	int count = 0;
-	while (items->name != NULL) {
+	while (items->name != 0) {
 	    if (!strcmp(dialog_vars.default_item, items->name)) {
 		result = count;
 		break;
@@ -490,18 +489,16 @@ dlg_form(const char *title,
 #ifdef KEY_RESIZE
     int old_height = height;
     int old_width = width;
-    int old_fhigh = form_height;
 #endif
 
     int form_width;
-    bool first = TRUE;
-    bool first_trace = TRUE;
+    int first = TRUE;
+    int first_trace = TRUE;
     int chr_offset = 0;
-    int state = (dialog_vars.default_button >= 0
-		 ? dlg_default_button()
-		 : sTEXT);
+    int state = dialog_vars.default_button >= 0 ? dlg_default_button() : sTEXT;
     int x, y, cur_x, cur_y, box_x, box_y;
     int code;
+    int key = 0;
     int fkey;
     int choice = dlg_default_formitem(items);
     int new_choice, new_scroll;
@@ -514,32 +511,19 @@ dlg_form(const char *title,
     bool field_changed = FALSE;
     bool non_editable = FALSE;
     WINDOW *dialog, *form;
-    char *prompt;
+    char *prompt = dlg_strclone(cprompt);
     const char **buttons = dlg_ok_labels();
     DIALOG_FORMITEM *current;
 
-    DLG_TRACE(("# %sform args:\n", (dialog_vars.formitem_type
-				    ? "password"
-				    : "")));
-    DLG_TRACE2S("title", title);
-    DLG_TRACE2S("message", cprompt);
-    DLG_TRACE2N("height", height);
-    DLG_TRACE2N("width", width);
-    DLG_TRACE2N("lheight", form_height);
-    DLG_TRACE2N("llength", item_no);
-    /* FIXME dump the items[][] too */
-    DLG_TRACE2N("current", *current_item);
-
     make_FORM_ELTs(items, item_no, &min_height, &min_width);
     dlg_button_layout(buttons, &min_width);
     dlg_does_output();
+    dlg_tab_correct_str(prompt);
 
 #ifdef KEY_RESIZE
   retry:
 #endif
 
-    prompt = dlg_strclone(cprompt);
-    dlg_tab_correct_str(prompt);
     dlg_auto_size(title, prompt, &height, &width,
 		  1 + 3 * MARGIN,
 		  MAX(26, 2 + min_width));
@@ -574,7 +558,7 @@ dlg_form(const char *title,
     dlg_draw_bottom_box2(dialog, border_attr, border2_attr, dialog_attr);
     dlg_draw_title(dialog, title);
 
-    dlg_attrset(dialog, dialog_attr);
+    (void) wattrset(dialog, dialog_attr);
     dlg_print_autowrap(dialog, prompt, height, width);
 
     form_width = width - 6;
@@ -609,7 +593,6 @@ dlg_form(const char *title,
 
     while (result == DLG_EXIT_UNKNOWN) {
 	int edit = FALSE;
-	int key;
 
 	if (scroll_changed) {
 	    print_form(form, items, item_no, scrollamt, choice);
@@ -646,6 +629,7 @@ dlg_form(const char *title,
 	    if (field_changed)
 		chr_offset = 0;
 	    current = &items[choice];
+	    dialog_vars.max_input = current->text_ilen;
 	    dlg_item_help(current->help);
 	    dlg_show_string(form, current->text, chr_offset,
 			    form_active_text_attr,
@@ -659,9 +643,8 @@ dlg_form(const char *title,
 	}
 
 	key = dlg_mouse_wgetch((state == sTEXT) ? form : dialog, &fkey);
-	if (dlg_result_key(key, fkey, &result)) {
+	if (dlg_result_key(key, fkey, &result))
 	    break;
-	}
 
 	/* handle non-functionkeys */
 	if (!fkey) {
@@ -699,10 +682,6 @@ dlg_form(const char *title,
 		dlg_del_window(dialog);
 		result = (state >= 0) ? dlg_enter_buttoncode(state) : DLG_EXIT_OK;
 		continue;
-	    case DLGK_LEAVE:
-		if (state >= 0)
-		    result = dlg_ok_buttoncode(state);
-		break;
 
 	    case DLGK_GRID_LEFT:
 		if (state == sTEXT)
@@ -781,15 +760,14 @@ dlg_form(const char *title,
 
 #ifdef KEY_RESIZE
 	    case KEY_RESIZE:
-		dlg_will_resize(dialog);
 		/* reset data */
 		height = old_height;
 		width = old_width;
-		form_height = old_fhigh;
-		free(prompt);
-		_dlg_resize_cleanup(dialog);
-		dlg_unregister_window(form);
 		/* repaint */
+		dlg_clear();
+		dlg_del_window(dialog);
+		refresh();
+		dlg_mouse_free_regions();
 		goto retry;
 #endif
 	    default:
@@ -867,13 +845,9 @@ dlg_form(const char *title,
 	}
 
 	if (state == sTEXT) {	/* Input box selected */
-	    if (!is_readonly(current)) {
-		int save = dialog_vars.max_input;
-		dialog_vars.max_input = current->text_ilen;
+	    if (!is_readonly(current))
 		edit = dlg_edit_string(current->text, &chr_offset, key,
 				       fkey, first);
-		dialog_vars.max_input = save;
-	    }
 	    if (edit) {
 		dlg_show_string(form, current->text, chr_offset,
 				form_active_text_attr,
@@ -888,7 +862,6 @@ dlg_form(const char *title,
     }
 
     dlg_mouse_free_regions();
-    dlg_unregister_window(form);
     dlg_del_window(dialog);
     free(prompt);
 
@@ -903,7 +876,7 @@ void
 dlg_free_formitems(DIALOG_FORMITEM * items)
 {
     int n;
-    for (n = 0; items[n].name != NULL; ++n) {
+    for (n = 0; items[n].name != 0; ++n) {
 	if (items[n].name_free)
 	    free(items[n].name);
 	if (items[n].text_free)
@@ -938,7 +911,7 @@ dialog_form(const char *title,
 	    char **items)
 {
     int result;
-    int choice = 0;
+    int choice;
     int i;
     DIALOG_FORMITEM *listitems;
     DIALOG_VARS save_vars;
